@@ -15,18 +15,21 @@ from tempfile import TemporaryDirectory
 
 import pytest
 
-from ..core import Pooch, retrieve, download_action, stream_download
+from ..core import create, Pooch, retrieve, download_action, stream_download
 from ..utils import get_logger, temporary_file, os_cache
 from ..hashes import file_hash, hash_matches
 
 # Import the core module so that we can monkeypatch some functions
 from .. import core
-from ..downloaders import HTTPDownloader
+from ..downloaders import HTTPDownloader, FTPDownloader
 
 from .utils import (
     pooch_test_url,
+    data_over_ftp,
     pooch_test_figshare_url,
     pooch_test_zenodo_url,
+    pooch_test_zenodo_with_slash_url,
+    pooch_test_dataverse_url,
     pooch_test_registry,
     check_tiny_data,
     check_large_data,
@@ -39,6 +42,8 @@ REGISTRY = pooch_test_registry()
 BASEURL = pooch_test_url()
 FIGSHAREURL = pooch_test_figshare_url()
 ZENODOURL = pooch_test_zenodo_url()
+ZENODOURL_W_SLASH = pooch_test_zenodo_with_slash_url()
+DATAVERSEURL = pooch_test_dataverse_url()
 REGISTRY_CORRUPTED = {
     # The same data file but I changed the hash manually to a wrong one
     "tiny-data.txt": "098h0894dba14b12085eacb204284b97e362f4f3e5a5807693cc90ef415c1b2d"
@@ -134,7 +139,9 @@ def test_pooch_local(data_dir_mirror):
 
 @pytest.mark.network
 @pytest.mark.parametrize(
-    "url", [BASEURL, FIGSHAREURL, ZENODOURL], ids=["https", "figshare", "zenodo"]
+    "url",
+    [BASEURL, FIGSHAREURL, ZENODOURL, DATAVERSEURL],
+    ids=["https", "figshare", "zenodo", "dataverse"],
 )
 def test_pooch_custom_url(url):
     "Have pooch download the file from URL that is not base_url"
@@ -158,7 +165,9 @@ def test_pooch_custom_url(url):
 
 @pytest.mark.network
 @pytest.mark.parametrize(
-    "url", [BASEURL, FIGSHAREURL, ZENODOURL], ids=["https", "figshare", "zenodo"]
+    "url",
+    [BASEURL, FIGSHAREURL, ZENODOURL, DATAVERSEURL],
+    ids=["https", "figshare", "zenodo", "dataverse"],
 )
 def test_pooch_download(url):
     "Setup a pooch that has no local data and needs to download"
@@ -328,6 +337,60 @@ def test_pooch_update():
             assert log_file.getvalue() == ""
 
 
+def test_pooch_update_disallowed():
+    "Test that disallowing updates works."
+    with TemporaryDirectory() as local_store:
+        path = Path(local_store)
+        # Create a dummy version of tiny-data.txt that is different from the
+        # one in the remote storage
+        true_path = str(path / "tiny-data.txt")
+        with open(true_path, "w") as fin:
+            fin.write("different data")
+        # Setup a pooch in a temp dir
+        pup = Pooch(
+            path=path,
+            base_url=BASEURL,
+            registry=REGISTRY,
+            allow_updates=False,
+        )
+        with pytest.raises(ValueError):
+            pup.fetch("tiny-data.txt")
+
+
+def test_pooch_update_disallowed_environment():
+    "Test that disallowing updates works through an environment variable."
+    variable_name = "MYPROJECT_DISALLOW_UPDATES"
+    try:
+        os.environ[variable_name] = "False"
+        with TemporaryDirectory() as local_store:
+            path = Path(local_store)
+            # Create a dummy version of tiny-data.txt that is different from
+            # the one in the remote storage
+            true_path = str(path / "tiny-data.txt")
+            with open(true_path, "w") as fin:
+                fin.write("different data")
+            # Setup a pooch in a temp dir
+            pup = create(
+                path=path,
+                base_url=BASEURL,
+                registry=REGISTRY,
+                allow_updates=variable_name,
+            )
+            with pytest.raises(ValueError):
+                pup.fetch("tiny-data.txt")
+    finally:
+        os.environ.pop(variable_name)
+
+
+def test_pooch_create_base_url_no_trailing_slash():
+    """
+    Test if pooch.create appends a trailing slash to the base url if missing
+    """
+    base_url = "https://mybase.url"
+    pup = create(base_url=base_url, registry=None, path=DATA_DIR)
+    assert pup.base_url == base_url + "/"
+
+
 @pytest.mark.network
 def test_pooch_corrupted(data_dir_mirror):
     "Raise an exception if the file hash doesn't match the registry"
@@ -412,6 +475,14 @@ def test_pooch_load_registry_invalid_line():
         pup.load_registry(os.path.join(DATA_DIR, "registry-invalid.txt"))
 
 
+def test_pooch_load_registry_with_spaces():
+    "Should check that spaces in filenames are allowed in registry files"
+    pup = Pooch(path="", base_url="")
+    pup.load_registry(os.path.join(DATA_DIR, "registry-spaces.txt"))
+    assert "file with spaces.txt" in pup.registry
+    assert "other with spaces.txt" in pup.registry
+
+
 @pytest.mark.network
 def test_check_availability():
     "Should correctly check availability of existing and non existing files"
@@ -428,21 +499,38 @@ def test_check_availability():
     assert not pup.is_available("not-a-real-data-file.txt")
 
 
-@pytest.mark.network
-def test_check_availability_on_ftp():
+def test_check_availability_on_ftp(ftpserver):
     "Should correctly check availability of existing and non existing files"
-    # Check available remote file on FTP server
-    pup = Pooch(
-        path=DATA_DIR,
-        base_url="ftp://data-out.unavco.org/pub/products/velocity/rel_201712/",
-        registry={
-            "pbo.final_igs08.20171202.vel": "md5:0b75d4049dedd0e179615f4b5e956156",
-            "doesnot_exist.zip": "jdjdjdjdflld",
-        },
-    )
-    assert pup.is_available("pbo.final_igs08.20171202.vel")
-    # Check non available remote file
-    assert not pup.is_available("doesnot_exist.zip")
+    with data_over_ftp(ftpserver, "tiny-data.txt") as url:
+        # Check available remote file on FTP server
+        pup = Pooch(
+            path=DATA_DIR,
+            base_url=url.replace("tiny-data.txt", ""),
+            registry={
+                "tiny-data.txt": "baee0894dba14b12085eacb204284b97e362f4f3e5a5807693cc90ef415c1b2d",
+                "doesnot_exist.zip": "jdjdjdjdflld",
+            },
+        )
+        downloader = FTPDownloader(port=ftpserver.server_port)
+        assert pup.is_available("tiny-data.txt", downloader=downloader)
+        # Check non available remote file
+        assert not pup.is_available("doesnot_exist.zip", downloader=downloader)
+
+
+def test_check_availability_invalid_downloader():
+    "Should raise an exception if the downloader doesn't support this"
+
+    def downloader(url, output, pooch):  # pylint: disable=unused-argument
+        "A downloader that doesn't support check_only"
+        return None
+
+    pup = Pooch(path=DATA_DIR, base_url=BASEURL, registry=REGISTRY)
+    # First check that everything works without the custom downloader
+    assert pup.is_available("tiny-data.txt")
+    # Now use the bad one
+    with pytest.raises(NotImplementedError) as error:
+        pup.is_available("tiny-data.txt", downloader=downloader)
+        assert "does not support availability checks" in str(error)
 
 
 @pytest.mark.network
@@ -537,3 +625,56 @@ def test_stream_download(fname):
         stream_download(url, destination, known_hash, downloader, pooch=None)
         assert destination.exists()
         check_tiny_data(str(destination))
+
+
+@pytest.mark.parametrize(
+    "url",
+    [FIGSHAREURL, ZENODOURL, DATAVERSEURL],
+    ids=["figshare", "zenodo", "dataverse"],
+)
+def test_load_registry_from_doi(url):
+    """Check that the registry is correctly populated from the API"""
+    with TemporaryDirectory() as local_store:
+        path = os.path.abspath(local_store)
+        pup = Pooch(path=path, base_url=url)
+        pup.load_registry_from_doi()
+
+        # Check the existence of all files in the registry
+        assert len(pup.registry) == 2
+        assert "tiny-data.txt" in pup.registry
+        assert "store.zip" in pup.registry
+
+        # Ensure that all files have correct checksums by fetching them
+        for filename in pup.registry:
+            pup.fetch(filename)
+
+
+def test_load_registry_from_doi_zenodo_with_slash():
+    """
+    Check that the registry is correctly populated from the Zenodo API when
+    the filename contains a slash
+    """
+    url = ZENODOURL_W_SLASH
+    with TemporaryDirectory() as local_store:
+        path = os.path.abspath(local_store)
+        pup = Pooch(path=path, base_url=url)
+        pup.load_registry_from_doi()
+
+        # Check the existence of all files in the registry
+        assert len(pup.registry) == 1
+        assert "santisoler/pooch-test-data-v1.zip" in pup.registry
+
+        # Ensure that all files have correct checksums by fetching them
+        for filename in pup.registry:
+            pup.fetch(filename)
+
+
+def test_wrong_load_registry_from_doi():
+    """Check that non-DOI URLs produce an error"""
+
+    pup = Pooch(path="", base_url=BASEURL)
+
+    with pytest.raises(ValueError) as exc:
+        pup.load_registry_from_doi()
+
+    assert "only implemented for DOIs" in str(exc.value)
