@@ -48,8 +48,8 @@ def pooch_test_url():
     """
     Get the base URL for the test data used in Pooch itself.
 
-    The URL is a github raw link to the ``pooch/tests/data`` directory from the
-    `Github repository <https://github.com/fatiando/pooch>`__. It matches the
+    The URL is a GitHub raw link to the ``pooch/tests/data`` directory from the
+    `GitHub repository <https://github.com/fatiando/pooch>`__. It matches the
     pooch version specified in ``pooch.version.full_version``.
 
     Returns
@@ -59,7 +59,7 @@ def pooch_test_url():
 
     """
     url = "https://github.com/fatiando/pooch/raw/{version}/pooch/tests/data/".format(
-        version=check_version(full_version)
+        version=check_version(full_version, fallback="main")
     )
     return url
 
@@ -98,6 +98,37 @@ def pooch_test_zenodo_url():
     return url
 
 
+def pooch_test_zenodo_with_slash_url():
+    """
+    Get base URL for test data in Zenodo, where the file name contains a slash
+
+    The URL contains the DOI for the Zenodo dataset that has a slash in the
+    filename (created with the GitHub-Zenodo integration service), using the
+    appropriate version for this version of Pooch.
+
+    Returns
+    -------
+    url
+        The URL for pooch's test data.
+
+    """
+    url = "doi:10.5281/zenodo.7632643/"
+    return url
+
+
+def pooch_test_dataverse_url():
+    """
+    Get the base URL for the test data stored on a DataVerse instance.
+
+    Returns
+    -------
+    url
+        The URL for pooch's test data.
+    """
+    url = "doi:10.11588/data/TKCFEF/"
+    return url
+
+
 def pooch_test_registry():
     """
     Get a registry for the test data used in Pooch itself.
